package com.example.sbvue.product;

import hello.service.MyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/api")
public class ProductController {

    private ProductRepository productRepository;
    private MyService myService;

    @Autowired
    public ProductController(ProductRepository productRepository, MyService myService) {
        this.productRepository = productRepository;
        this.myService = myService;
    }

    @RequestMapping("/")
    @CrossOrigin("*")
    public String getHome() {
        System.out.println("getProducts(), myService.message=" + myService.message());
        return "Hello World...";
    }

    @RequestMapping("/product")
    @CrossOrigin("*")
    public List<Product> getProducts() {
        System.out.println("getProducts(), myService.message=" + myService.message());
        return productRepository.findAll();
    }

    @RequestMapping("/product/{id}")
    @CrossOrigin("*")
    public Product getProductById(@PathVariable("id") int id) {
        System.out.println("getProductById(), myService.message=" + myService.message());
        /*
        // 1. Non error handling
        return productRepository.findById(id).get();
        */

        /*
        // 2. Use isPresent()
        Optional<Product> product = productRepository.findById(id);
        if (!product.isPresent()) {
            return new Product();
        }
        return product.get();
        */

        /*
        // 3. Use Supplier
        return productRepository.findById(id).orElseGet(new Supplier<Product>() {
            @Override
            public Product get() {
                return new Product();
            }
        });
        */

        // 4. Use Method Reference
        return productRepository.findById(id).orElseGet(Product::new);
    }
}
